#!/usr/bin/env python3
'''
Extract data from AddressBook database (.abcddb) to Contacts VCards file (.vcf)
'''
import os
import sys
from io import TextIOWrapper
from pathlib import Path
from argparse import ArgumentParser
try:
    from .ABCDDB import ABCDDB, Record
except ImportError:  # fallback if not run as module
    from ABCDDB import ABCDDB, Record  # type: ignore[import, no-redef]

DB_FILE = str(Path.home().joinpath(
    'Library', 'Application Support', 'AddressBook', 'AddressBook-v22.abcddb'))


def main() -> None:
    cli = ArgumentParser(description=__doc__)
    cli.add_argument('output', type=str, metavar='outfile.vcf',
                     help='VCard output file.')
    cli.add_argument('-f', '--force', action='store_true',
                     help='Overwrite existing output file.')
    cli.add_argument('--dry-run', action='store_true',
                     help='Do not write file(s), just print filenames.')
    cli.add_argument('-i', '--input', type=str, metavar='AddressBook.abcddb',
                     default=DB_FILE, help='Specify another abcddb input file.'
                     ' Default: ' + DB_FILE)
    cli.add_argument('-s', '--split', type=str, metavar='FORMAT', help='''
        Output into several vcf files instead of a single file.
        File format can use any field of type Record.
        E.g. "%%{id}_%%{fullname}.vcf".
    ''')
    args = cli.parse_args()

    # check input args
    if not os.path.isfile(args.input):
        print('AddressBook "{}" does not exist.'.format(args.input),
              file=sys.stderr)
        exit(1)
    elif not os.path.isdir(os.path.dirname(args.output) or os.curdir):
        print('Output parent directory does not exist.', file=sys.stderr)
        exit(1)
    elif os.path.exists(args.output) and not args.force:
        print('Output file already exist. Use -f to force overwrite.',
              file=sys.stderr)
        exit(1)

    # perform export
    contacts = ABCDDB.load(args.input)
    export_count = 0

    # reused for appending to an open file
    def writeRec(f: TextIOWrapper, rec: Record):
        nonlocal export_count
        try:
            f.write(rec.makeVCard())
            export_count += 1
        except Exception as e:
            print(f'Error processing contact {rec.id} {rec.fullname}: {e}',
                  file=sys.stderr)

    # choose which export mode to use
    if args.split:  # multi-file mode
        outDir = Path(args.output)
        prevFilenames = set()
        for rec in contacts:
            filename = outDir / Path(rec.formatFilename(args.split))
            if filename in prevFilenames:
                print(f'WARN: overwriting "{filename}"', file=sys.stderr)
            prevFilenames.add(filename)

            os.makedirs(filename.parent, exist_ok=True)
            if args.dry_run:
                print(filename)
            else:
                with open(filename, 'w') as f:
                    writeRec(f, rec)
    else:  # single-file mode
        if args.dry_run:
            print(args.output)
        else:
            with open(args.output, 'w') as f:
                for rec in contacts:
                    writeRec(f, rec)
    print(f'{export_count}/{len(contacts)} contacts.')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
from setuptools import setup
from abcddb2vcard import __doc__, __version__

with open('README.md') as fp:
    longdesc = fp.read()

setup(
    name='abcddb2vcard',
    description=__doc__.strip(),
    version=__version__,
    author='relikd',
    url='https://github.com/relikd/abcddb2vcard',
    license='MIT',
    packages=['abcddb2vcard'],
    entry_points={
        'console_scripts': [
            'abcddb2vcard=abcddb2vcard.abcddb2vcard:main',
            'vcard2img=abcddb2vcard.vcard2img:main',
        ]
    },
    long_description_content_type="text/markdown",
    long_description=longdesc,
    python_requires='>=3.6',
    keywords=[
        'abcddb',
        'abcd',
        'address book',
        'vcard',
        'contacts',
        'converter',
        'export',
        'backup',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: MacOS X',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: SQL',
        'Topic :: Communications :: Email :: Address Book',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Archiving :: Backup',
        'Topic :: Utilities',
    ],
)



#!/usr/bin/env python3
'''
Extract all profile pictures from a Contacts VCards file (.vcf)
'''
import os
import sys
from base64 import b64decode
from argparse import ArgumentParser, FileType


def main() -> None:
    cli = ArgumentParser(description=__doc__)
    cli.add_argument('input', type=FileType('r'), metavar='infile.vcf',
                     help='VCard input file.')
    cli.add_argument('outdir', type=str, help='Output directory.')
    args = cli.parse_args()

    # check input args
    if not os.path.isdir(os.path.dirname(args.outdir) or os.curdir):
        print('Output parent directory does not exist.', file=sys.stderr)
        exit(1)

    os.makedirs(args.outdir, exist_ok=True)

    # perform export
    c1 = 0
    c2 = 0
    name = ''
    img = ''
    collect = False
    for line in args.input.readlines():
        line = line.rstrip()
        if line == 'BEGIN:VCARD':
            c1 += 1
            name = ''
            img = ''
            collect = False
        elif line.startswith('FN:'):
            name = line.split(':', 1)[1]
        elif line.startswith('PHOTO;'):
            img = line.split(':', 1)[1]
            collect = True
        elif collect:
            if line[0] == ' ':
                img += line[1:]
            else:
                collect = False
        if line == 'END:VCARD' and img:
            c2 += 1
            name = name.replace('\\,', ',').replace('\\;', ';').replace(
                '/', '-')
            with open(os.path.join(args.outdir, name + '.jpg'), 'wb') as fw:
                fw.write(b64decode(img))

    print(c1, 'contacts.', c2, 'images.')


if __name__ == '__main__':
    main()
